@@ -1,23 +1,36 @@
 # This script generates the PyQt configuration and generates the Makefiles.
 #
-# Copyright (c) 2007
-# 	Phil Thompson <phil@river-bank.demon.co.uk>
+# Copyright (c) 2008 Riverbank Computing Limited <info@riverbankcomputing.com>
 # 
 # This file is part of PyQt.
 # 
-# This copy of PyQt is free software; you can redistribute it and/or modify it
-# under the terms of the GNU General Public License version 2 as published by
-# the Free Software Foundation and appearing in the file LICENSE included in the
-# packaging of this file.
+# This file may be used under the terms of the GNU General Public
+# License versions 2.0 or 3.0 as published by the Free Software
+# Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
+# included in the packaging of this file.  Alternatively you may (at
+# your option) use any later version of the GNU General Public
+# License if such license has been publicly approved by Riverbank
+# Computing Limited (or its successors, if any) and the KDE Free Qt
+# Foundation. In addition, as a special exception, Riverbank gives you
+# certain additional rights. These rights are described in the Riverbank
+# GPL Exception version 1.1, which can be found in the file
+# GPL_EXCEPTION.txt in this package.
 # 
-# PyQt is supplied in the hope that it will be useful, but WITHOUT ANY
-# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
-# details.
+# Please review the following information to ensure GNU General
+# Public Licensing requirements will be met:
+# http://trolltech.com/products/qt/licenses/licensing/opensource/. If
+# you are unsure which license is appropriate for your use, please
+# review the following information:
+# http://trolltech.com/products/qt/licenses/licensing/licensingoverview
+# or contact the sales department at sales@riverbankcomputing.com.
 # 
-# You should have received a copy of the GNU General Public License along with
-# PyQt; see the file LICENSE.  If not, write to the Free Software Foundation,
-# Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# This file is provided "AS IS" with NO WARRANTY OF ANY KIND,
+# INCLUDING THE WARRANTIES OF DESIGN, MERCHANTABILITY AND FITNESS FOR
+# A PARTICULAR PURPOSE. Trolltech reserves all rights not expressly
+# granted herein.
+# 
+# This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
+# WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 
 
 import sys
@@ -31,10 +44,10 @@ import sipconfig
 
 
 # Initialise the globals.
-pyqt_version = 0x040303
-pyqt_version_str = "4.3.3"
+pyqt_version = 0x040402
+pyqt_version_str = "4.4.2"
 
-sip_min_version = 0x040702
+sip_min_version = 0x040705
 
 qt_version = 0
 qt_edition = ""
@@ -250,7 +263,8 @@ class ConfigurePyQt4:
             0x040202: "Qt_4_2_0",
             0x040300: "Qt_4_2_2",
             0x040303: "Qt_4_3_0",
-            0x050000: "Qt_4_3_3"
+            0x040400: "Qt_4_3_3",
+            0x050000: "Qt_4_4_0"
         }
 
     def check_modules(self):
@@ -271,6 +285,7 @@ class ConfigurePyQt4:
         pyqt_modules.append("QtCore")
 
         check_module("QtGui", "qwidget.h", "new QWidget()")
+        check_module("QtHelp", "qhelpengine.h", "new QHelpEngine(\"foo\")")
         check_module("QtNetwork", "qhostaddress.h", "new QHostAddress()")
         check_module("QtOpenGL", "qgl.h", "new QGLWidget()")
         check_module("QtScript", "qscriptengine.h", "new QScriptEngine()")
@@ -278,7 +293,10 @@ class ConfigurePyQt4:
                 extra_libs=sql_libs)
         check_module("QtSvg", "qsvgwidget.h", "new QSvgWidget()")
         check_module("QtTest", "QtTest", "QTest::qSleep(0)")
+        check_module("QtWebKit", "qwebpage.h", "new QWebPage()")
         check_module("QtXml", "qdom.h", "new QDomDocument()")
+        check_module("QtXmlPatterns", "qxmlname.h", "new QXmlName()")
+        check_module("phonon", "phonon", "new Phonon::VideoWidget()")
         check_module("QtAssistant", "qassistantclient.h",
                 "new QAssistantClient(\"foo\")", extra_lib_dirs=ass_lib_dirs,
                 extra_libs=ass_libs)
@@ -332,6 +350,9 @@ class ConfigurePyQt4:
                 generate_code("QtGui", extra_lib_dirs=sp_libdirs,
                         extra_libs=sp_libs)
 
+        if "QtHelp" in pyqt_modules:
+            generate_code("QtHelp")
+
         if "QtNetwork" in pyqt_modules:
             generate_code("QtNetwork")
 
@@ -359,9 +380,18 @@ class ConfigurePyQt4:
         if "QtTest" in pyqt_modules:
             generate_code("QtTest")
 
+        if "QtWebKit" in pyqt_modules:
+            generate_code("QtWebKit")
+
         if "QtXml" in pyqt_modules:
             generate_code("QtXml")
 
+        if "QtXmlPatterns" in pyqt_modules:
+            generate_code("QtXmlPatterns")
+
+        if "phonon" in pyqt_modules:
+            generate_code("phonon")
+
         if "QtAssistant" in pyqt_modules:
             generate_code("QtAssistant")
 
@@ -865,7 +895,7 @@ def run_command(cmd):
     if opts.verbose:
         sys.stdout.write(cmd + "\n")
 
-    fin, fout = os.popen4(cmd)
+    fout = get_command_stdout(cmd, and_stderr=True)
 
     # Read stdout and stderr until there is no more output.
     lout = fout.readline()
@@ -875,7 +905,6 @@ def run_command(cmd):
 
         lout = fout.readline()
 
-    fin.close()
     fout.close()
 
     try:
@@ -909,7 +938,11 @@ def compile_qt_program(name, mname, extra_include_dirs=None, extra_lib_dirs=None
     """
     opengl = (mname == "QtOpenGL")
 
-    makefile = sipconfig.ProgramMakefile(sipcfg, console=1, qt=[mname], warnings=0, opengl=opengl, debug=opts.debug)
+    qt = [mname]
+    if mname == "QtWebKit":
+        qt.append("QtCore")
+
+    makefile = sipconfig.ProgramMakefile(sipcfg, console=1, qt=qt, warnings=0, opengl=opengl, debug=opts.debug)
 
     add_makefile_extras(makefile, extra_include_dirs, extra_lib_dirs, extra_libs)
 
@@ -970,12 +1003,36 @@ def check_vendorid():
                              "correct directory." % opts.vendincdir)
 
 
+def get_command_stdout(cmd, and_stderr=False):
+    """Return stdout (and optionally stderr) from the given command.
+    """
+    try:
+        import subprocess
+    except ImportError:
+        if and_stderr:
+            _, sout = os.popen4(cmd)
+        else:
+            _, sout, _ = os.popen3(cmd)
+
+        return sout
+
+    if and_stderr:
+        stderr = subprocess.STDOUT
+    else:
+        stderr = subprocess.PIPE
+
+    p = subprocess.Popen(cmd, shell=True, stdin=subprocess.PIPE,
+            stdout=subprocess.PIPE, stderr=stderr)
+
+    return p.stdout
+
+
 def check_dbus():
     """See if the DBus support module should be built.
     """
     sipconfig.inform("Checking to see if the dbus support module should be built...")
 
-    _, sout, _ = os.popen3("pkg-config --cflags-only-I --libs dbus-1")
+    sout = get_command_stdout("pkg-config --cflags-only-I --libs dbus-1")
     iflags = sout.read().strip()
 
     if not iflags:
@@ -1134,17 +1191,22 @@ def needed_qt_libs(mname, qt_libs):
     """
 
     # The dependencies between the different Qt libraries.  The order within
-    # each list is important.
+    # each list is important.  Note that this affects the include directories
+    # as well as the libraries.
     LIB_DEPS = {
         "QtCore": [],
         "QtGui": ["QtCore"],
+        "QtHelp": ["QtGui"],
         "QtNetwork": ["QtCore"],
         "QtOpenGL": ["QtGui"],
         "QtScript": ["QtCore"],
         "QtSql": ["QtGui"],
         "QtSvg": ["QtGui"],
         "QtTest": ["QtGui"],
+        "QtWebKit": ["QtNetwork", "QtGui"],
         "QtXml": ["QtCore"],
+        "QtXmlPatterns": ["QtCore"],
+        "phonon": ["QtGui"],
         "QtAssistant": ["QtNetwork", "QtGui"],
         "QtDesigner": ["QtGui"],
         "QAxContainer": ["QtGui"]
@@ -1315,13 +1377,19 @@ def check_license():
 
     # Common checks.
     if qt_edition and ltype != "internal":
-        if (qt_edition == "free" and ltype != "GPL") or (qt_edition != "free" and ltype == "GPL"):
+        if qt_edition != "free" and ltype == "GPL":
             sipconfig.error("This version of PyQt and the %s edition of Qt have incompatible licenses." % qt_edition)
 
     # Confirm the license if not already done.
     if not opts.license_confirmed:
         print
-        print "Type 'L' to view the license."
+
+        if ltype == "GPL":
+            print "Type '2' to view the GPL v2 license."
+            print "Type '3' to view the GPL v3 license."
+        else:
+            print "Type 'L' to view the license."
+
         print "Type 'yes' to accept the terms of the license."
         print "Type 'no' to decline the terms of the license."
         print
@@ -1342,8 +1410,14 @@ def check_license():
             if resp == "no":
                 sys.exit(0)
 
-            if resp == "l":
-                os.system("more LICENSE")
+            if ltype == "GPL":
+                if resp == "2":
+                    os.system("more LICENSE.GPL2")
+                elif resp == "3":
+                    os.system("more LICENSE.GPL3")
+            else:
+                if resp == "l":
+                    os.system("more LICENSE")
 
     # If there should be a license file then check it is where it should be.
     if lfile:
@@ -1567,7 +1641,7 @@ int main(int, char **)
     run_command("%s -f %s%s" % (make, make_file, make_target))
 
     if not os.access(exe_file, os.X_OK):
-        sipconfig.error("Failed to determine the layout of your Qt installtion. Try again using the --verbose flag to see more detail about the problem.")
+        sipconfig.error("Failed to determine the layout of your Qt installation. Try again using the --verbose flag to see more detail about the problem.")
 
     # Create the output file, first making sure it doesn't exist.
     remove_file(out_file)
@@ -1630,15 +1704,12 @@ int main(int, char **)
             sipconfig.error("Qt has been built as static libraries so either the -g or -k argument should be used.")
 
 
-def main(argv):
+def main():
     """Create the configuration module module.
-
-    argv is the list of command line arguments.
     """
     # Check SIP is new enough.
-    if "snapshot" not in sipcfg.sip_version_str:
-        if sipcfg.sip_version < sip_min_version:
-            sipconfig.error("This version of PyQt requires SIP v%s or later" % sipconfig.version_to_string(sip_min_version))
+    if sipcfg.sip_version < sip_min_version:
+        sipconfig.error("This version of PyQt requires SIP v%s or later" % sipconfig.version_to_string(sip_min_version))
 
     global opts
 
@@ -1737,12 +1808,12 @@ def main(argv):
 
 if __name__ == "__main__":
     try:
-        main(sys.argv)
+        main()
     except SystemExit:
         raise
     except:
         print \
 """An internal error occured.  Please report all the output from the program,
-including the following traceback, to support@riverbankcomputing.co.uk.
+including the following traceback, to support@riverbankcomputing.com.
 """
         raise
