@@ -25,7 +25,7 @@
 import sys
 import re
 
-from PyQt4 import QtCore, QtGui, QtNetwork, QtXml, QtAssistant
+from PyQt4 import QtCore, QtGui, QtHelp, QtNetwork, QtXml
 
 from displaywidget import DisplayWidget
 from displayshape import TitleShape, DisplayShape, PanelShape, ImageShape, DocumentShape
@@ -47,6 +47,7 @@ class Launcher(QtGui.QMainWindow):
         self.qtLogo = QtGui.QImage()
         self.rbLogo = QtGui.QImage()
         self.currentExample = ""
+        self.assistantProcess = QtCore.QProcess()
 
         parentPageAction1 = QtGui.QAction(self.tr("Show Parent Page"), self)
         parentPageAction2 = QtGui.QAction(self.tr("Show Parent Page"), self)
@@ -81,8 +82,6 @@ class Launcher(QtGui.QMainWindow):
         self.resizeTimer.setSingleShot(True)
         self.connect(self.resizeTimer, QtCore.SIGNAL("timeout()"), self.redisplayWindow)
 
-        self.assistant = QtAssistant.QAssistantClient("assistant", self)
-
         self.connect(self.display, QtCore.SIGNAL("actionRequested"),
                      self.executeAction)
         self.connect(self.display, QtCore.SIGNAL("categoryRequested"),
@@ -105,21 +104,53 @@ class Launcher(QtGui.QMainWindow):
         self.setWindowTitle(self.tr("PyQt Examples and Demos"))
         self.setWindowIcon(QtGui.QIcon(QtGui.QPixmap(":/images/qt4-logo.png")))
 
+    def initHelpEngine(self):
+        self.helpRootUrl = QtCore.QString("qthelp://com.trolltech.qt.%d%d%d/qdoc/" % (QtCore.QT_VERSION >> 16, ((QtCore.QT_VERSION >> 8) & 0xff), (QtCore.QT_VERSION & 0xff)))
+
+        # Store help collection file in cache dir of assistant.
+        cacheDir = QtGui.QDesktopServices.storageLocation(QtGui.QDesktopServices.DataLocation) + "/Trolltech/Assistant"
+        helpDataFile = "qtdemo_%s.qhc" % QtCore.QT_VERSION_STR
+
+        dir = QtCore.QDir()
+        if not dir.exists(cacheDir):
+            dir.mkpath(cacheDir)
+
+        # Create the help engine (and a new helpDataFile if it does not exist).
+        self.helpEngine = QtHelp.QHelpEngineCore(cacheDir + helpDataFile)
+        self.helpEngine.setupData()
+
+        qtDocRoot = QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.DocumentationPath) + "/qch"
+        qtDocRoot = QtCore.QDir(qtDocRoot).absolutePath()
+
+        qchFiles = ["/qt.qch", "/designer.qch", "/linguist.qch"]
+
+        oldDir = self.helpEngine.customValue("docDir", QtCore.QVariant(QtCore.QString())).toString()
+        if oldDir != qtDocRoot:
+            for qchFile in qchFiles:
+                self.helpEngine.unregisterDocumentation(QtHelp.QHelpEngineCore.namespaceName(qtDocRoot + qchFile))
+
+        # If the data that the engine will work on is not yet registered, do it
+        # now.
+        for qchFile in qchFiles:
+            self.helpEngine.registerDocumentation(qtDocRoot + qchFile)
+
+        self.helpEngine.setCustomValue("docDir", QtCore.QVariant(qtDocRoot))
+
     def setup(self):
+        self.initHelpEngine()
+
         self.documentationDir = QtCore.QDir(
                                         QtCore.QLibraryInfo.location(
                                                 QtCore.QLibraryInfo.DocumentationPath))
 
-        if not self.documentationDir.cd("html"):
-            QtGui.QMessageBox.warning(self, self.tr("No Documentation Found"),
-                            self.tr("I could not find the Qt documentation."),
-                            QtGui.QMessageBox.Cancel, QtGui.QMessageBox.NoButton)
-
         self.imagesDir = QtCore.QDir(self.documentationDir)
-        if not self.imagesDir.cd("images"):
-            QtGui.QMessageBox.warning(self, self.tr("No Images found"),
-                            self.tr("I could not find any images for the Qt documentation"),
-                            QtGui.QMessageBox.Cancel, QtGui.QMessageBox.NoButton)
+
+        if self.documentationDir.cd("html") and self.documentationDir.cd("images"):
+            self.imagesDir.setPath(self.documentationDir.path())
+            self.documentationDir.cdUp()
+        else:
+            QtGui.QMessageBox.warning(self, self.tr("No Documentation Found"),
+                    self.tr("I could not find the Qt documentation."))
 
         self.maximumLabels = 0
 
@@ -353,7 +384,26 @@ class Launcher(QtGui.QMainWindow):
         self.disconnect(self.display, QtCore.SIGNAL("displayEmpty()"), self.close)
         self.currentExample = uniqueName
 
-        self.assistant.showPage(self.examples[uniqueName]["document path"])
+        url = self.helpRootUrl + self.examples[uniqueName]["document path"]
+
+        # Start assistant if it isn't already running.
+        if self.assistantProcess.state() != QtCore.QProcess.Running:
+            app = QtCore.QLibraryInfo.location(QtCore.QLibraryInfo.BinariesPath) + QtCore.QDir.separator()
+            if sys.platform == 'darwin':
+                app += "Assistant.app/Contents/MacOS/Assistant"
+            else:
+                app += "assistant"
+
+            self.assistantProcess.start(app, ["-enableRemoteControl"])
+
+            if not self.assistantProcess.waitForStarted():
+                QtGui.QMessageBox.critical(None, "Qt Demo", "Could not start Qt Assistant.")
+                return
+
+        # Send command through remote control even if the process was started
+        # to activate assistant and bring it to front.
+        str = QtCore.QTextStream(self.assistantProcess)
+        str << "SetSource " << url << '\n'
 
     def showExamples(self, category):
         self.newPage()
@@ -582,7 +632,7 @@ class Launcher(QtGui.QMainWindow):
 
             rightMargin = background.rect().left()
 
-        if self.examples[self.currentExample].has_key('document path'):
+        if self.examples[self.currentExample]['document path']:
 
             documentCaption = TitleShape(self.tr("Show Documentation"),
                 self.font(), QtGui.QPen(QtCore.Qt.white), QtCore.QPointF(0.0, 0.0), QtCore.QSizeF(maxSize),
@@ -876,11 +926,11 @@ class Launcher(QtGui.QMainWindow):
 
     def findDescriptionAndImages(self, uniqueName, docName):
         if self.documentationDir.exists(docName):
-            self.examples[uniqueName]['document path'] = self.documentationDir.absoluteFilePath(docName)
+            self.examples[uniqueName]['document path'] = docName
 
             exampleDoc = QtXml.QDomDocument()
 
-            exampleFile = QtCore.QFile(self.examples[uniqueName]['document path'])
+            exampleFile = QtCore.QFile(self.documentationDir.absoluteFilePath(docName))
             exampleDoc.setContent(exampleFile)
 
             paragraphs = exampleDoc.elementsByTagName("p")
