@@ -1,10 +1,7 @@
-import logging
-
 from PyQt4 import QtGui, QtCore
 from PyQt4.uic import uiparser
 from qobjectcreator import LoaderCreatorPolicy
 
-DEBUG = logging.debug
 
 class DynamicUILoader(uiparser.UIParser):
     def __init__(self):
