@@ -18,7 +18,12 @@ class _ModuleWrapper(object):
         if cls in self._classes:
             if self._module is None:
                 self._module = __import__(self._moduleName, {}, {}, self._classes)
-            return getattr(self._module, cls)
+            # Allow for namespaces.
+            obj = self._module
+            for attr in cls.split('.'):
+                obj = getattr(obj, attr)
+
+            return obj
         else:
             return None
 
