@@ -4,6 +4,10 @@ import string
 
 from PyQt4.uic.exceptions import UnsupportedPropertyError
 
+
+logger = logging.getLogger(__name__)
+DEBUG = logger.debug
+
 QtCore = None
 QtGui = None
 
@@ -19,6 +23,9 @@ def ascii_upper(s):
 def int_list(prop):
     return [int(child.text) for child in prop]
 
+def float_list(prop):
+    return [float(child.text) for child in prop]
+
 bool_ = lambda v: v == "true"
 
 def needsWidget(func):
@@ -40,7 +47,7 @@ class Properties(object):
     def _pyEnumMember(self, cpp_name):
         try:
             prefix, membername = cpp_name.split("::")
-            logging.debug(membername)
+            DEBUG(membername)
             if prefix == "Qt":
                 return getattr(QtCore.Qt, membername)
             else:
@@ -58,21 +65,29 @@ class Properties(object):
     def _number(self, prop):
         return int(prop.text)
 
+    _uInt = _longLong = _uLongLong = _number
+
     def _double(self, prop):
         return float(prop.text)
 
     def _bool(self, prop):
         return prop.text == 'true'
 
+    def _stringlist(self, prop):
+        return [self._string(p) for p in prop]
+
     def _string(self, prop):
         if prop.get("notr", None) == "true":
             return self._cstring(prop)
-        else:
-            if prop.text is None:
-                return ""
-            text = prop.text.encode("UTF-8")
-            return QtGui.QApplication.translate(self.uiname, text,
-                                                None, QtGui.QApplication.UnicodeUTF8)
+
+        if prop.text is None:
+            return ""
+
+        text = prop.text.encode("UTF-8")
+        return QtGui.QApplication.translate(self.uiname, text, None,
+                QtGui.QApplication.UnicodeUTF8)
+
+    _char = _string
 
     def _cstring(self, prop):
         return str(prop.text)
@@ -88,17 +103,53 @@ class Properties(object):
 
         return QtGui.QColor(*args)
 
+    def _point(self, prop):
+        return QtCore.QPoint(*int_list(prop))
+
+    def _pointf(self, prop):
+        return QtCore.QPointF(*float_list(prop))
+
     def _rect(self, prop):
         return QtCore.QRect(*int_list(prop))
 
+    def _rectf(self, prop):
+        return QtCore.QRectF(*float_list(prop))
+
     def _size(self, prop):
         return QtCore.QSize(*int_list(prop))
 
+    def _sizef(self, prop):
+        return QtCore.QSizeF(*float_list(prop))
+
     def _pixmap(self, prop):
         return QtGui.QPixmap(prop.text.replace("\\", "\\\\"))
 
     def _iconset(self, prop):
-        return QtGui.QIcon(prop.text.replace("\\", "\\\\"))
+        # Qt v4.4.0 added support for specifying pixmaps for each of the icon's
+        # modes and states.
+        icon = self.factory.createQObject("QIcon", "icon", (), is_attribute=False)
+        no_pixmaps = True
+
+        for e in prop:
+            if e.tag.endswith("off"):
+                mode = e.tag[:-3]
+                state = QtGui.QIcon.Off
+            elif e.tag.endswith("on"):
+                mode = e.tag[:-2]
+                state = QtGui.QIcon.On
+            else:
+                continue
+
+            mode = getattr(QtGui.QIcon, mode.title())
+
+            icon.addPixmap(QtGui.QPixmap(e.text.replace("\\", "\\\\")), mode, state)
+            no_pixmaps = False
+
+        if no_pixmaps:
+            # Fallback to the old behaviour.
+            icon.addFile(prop.text.replace("\\", "\\\\"))
+
+        return icon
 
     def _url(self, prop):
         return QtCore.QUrl(prop[0].text)
@@ -226,7 +277,7 @@ class Properties(object):
             if prop[0].text is None:
                 continue
             propname = prop.attrib["name"]
-            logging.debug("setting property %s" % (propname,))
+            DEBUG("setting property %s" % (propname,))
 
             try:
                 stdset = bool(int(prop.attrib["stdset"]))
@@ -284,13 +335,13 @@ class Properties(object):
     def buddy(self, widget, prop):
         self.buddies.append((widget, prop[0].text))
 
-    # geometry has to be handled specially if set on the toplevel widget.
+    # geometry is handled specially if set on the toplevel widget.
     def geometry(self, widget, prop):
-        rect = self._rect(prop[0])
         if widget.objectName() == self.uiname:
-            widget.resize(QtCore.QSize(rect.size()).expandedTo(widget.minimumSizeHint()))
+            geom = int_list(prop[0])
+            widget.resize(geom[2], geom[3])
         else:
-            widget.setGeometry(rect)
+            widget.setGeometry(self._rect(prop[0]))
 
     def orientation(self, widget, prop):
         # If the class is a QFrame, it's a line.
