@@ -20,13 +20,15 @@ from exceptions import NoSuchWidgetError
 from properties import Properties
 from objcreator import QObjectCreator
 
+
+logger = logging.getLogger(__name__)
+DEBUG = logger.debug
+
 if sys.version_info < (2,4,0):
     def reversed(seq):
         for i in xrange(len(seq)-1, -1, -1):
             yield seq[i]
 
-DEBUG = logging.debug
-
 QtCore = None
 QtGui = None
 
@@ -134,7 +136,7 @@ class UIParser(object):
     
     def createWidget(self, elem):
         def widgetClass(elem):
-            cls = elem.attrib["class"]
+            cls = elem.attrib["class"].replace('::', '.')
             if cls == "Line":
                 return "QFrame"
             else:
@@ -296,7 +298,8 @@ class UIParser(object):
             parent = None
         else:
             parent = self.stack.topwidget
-        elem.attrib["name"] = classname[1:].lower()
+        if "name" not in elem.attrib:
+            elem.attrib["name"] = classname[1:].lower()
         self.stack.push(self.setupObject(classname, parent, elem))
         self.traverseWidgetTree(elem)
         layout = self.stack.popLayout()
